@@ -11,13 +11,15 @@ import dicom
 import os.path
 
 from pkg_resources import Requirement, resource_filename
-testcharset_dir = resource_filename(Requirement.parse("pydicom"),"dicom/testcharsetfiles")
+testcharset_dir = resource_filename(Requirement.parse("pydicom"),
+                                                    "dicom/testcharsetfiles")
 
 latin1_file = os.path.join(testcharset_dir, "chrFren.dcm")
 jp_file = os.path.join(testcharset_dir, "chrH31.dcm")
 multiPN_file = os.path.join(testcharset_dir, "chrFrenMulti.dcm")
+sq_encoding_file = os.path.join(testcharset_dir, "chrSQEncoding.dcm")
 
-test_dir = resource_filename(Requirement.parse("pydicom"),"dicom/testfiles")
+test_dir = resource_filename(Requirement.parse("pydicom"), "dicom/testfiles")
 normal_file = os.path.join(test_dir, "CT_small.dcm")
 
 
@@ -27,21 +29,38 @@ class charsetTests(unittest.TestCase):
         ds = dicom.read_file(latin1_file)
         ds.decode()
         # Make sure don't get unicode encode error on converting to string
-        expected = u"Buc^Jrme"
+        expected = u'Buc^J\xe9r\xf4me'
         got = ds.PatientName
-        self.assertEqual(expected, got, "Expected %r, got %r" % (expected, got))
+        self.assertEqual(expected, got,
+                                "Expected %r, got %r" % (expected, got))
+
+    def testNestedCharacterSets(self):
+        """charset: can read and decode SQ with different encodings........."""
+        ds = dicom.read_file(sq_encoding_file)
+        ds.decode()
+        # These datasets inside of the SQ cannot be decoded with default_encoding
+        # OR UTF-8 (the parent dataset's encoding). Instead, we make sure that it
+        # is decoded using the (0008,0005) tag of the dataset
+        expected = u'\uff94\uff8f\uff80\uff9e^\uff80\uff9b\uff73=\u5c71\u7530^\u592a\u90ce=\u3084\u307e\u3060^\u305f\u308d\u3046'
+        got = ds[0x32, 0x1064][0].PatientName
+        self.assertEqual(expected, got,
+                                "Expected %r, got %r" % (expected, got))
+
     def testStandardFile(self):
         """charset: can read and decode standard file without special char.."""
         ds = dicom.read_file(normal_file)
         ds.decode()
+
     def testMultiPN(self):
         """charset: can decode file with multi-valued data elements........."""
         ds = dicom.read_file(multiPN_file)
         ds.decode()
 
+
 if __name__ == "__main__":
     # This is called if run alone, but not if loaded through run_tests.py
-    # If not run from the directory where the sample images are, then need to switch there
+    # If not run from the directory where the sample images are,
+    #   then need to switch there
     import sys
     import os
     import os.path
@@ -51,4 +70,4 @@ if __name__ == "__main__":
         os.chdir(dir_name)
     os.chdir("../testfiles")
     unittest.main()
-    os.chdir(save_dir)
\ No newline at end of file
+    os.chdir(save_dir)
